@@ -100,15 +100,19 @@ public Q_SLOTS:
      * @map - NMVariantMapMap with new connection settings
      */
     void updateConnection(const NetworkManager::Connection::Ptr &connection, const NMVariantMapMap &map);
-    void requestScan();
+    void requestScan(const QString &interface = QString());
 
 private Q_SLOTS:
     void initKdedModule();
+    void secretAgentError(const QString &connectionPath, const QString &message);
     void replyFinished(QDBusPendingCallWatcher *watcher);
 #if WITH_MODEMMANAGER_SUPPORT
     void unlockRequiredChanged(MMModemLock modemLock);
 #endif
 
+Q_SIGNALS:
+    void wirelessScanTimerEnabled(bool enable);
+    void connectionActivationFailed(const QString &connectionPath, const QString &message);
 private:
     bool m_tmpWirelessEnabled;
     bool m_tmpWwanEnabled;
@@ -119,8 +123,10 @@ private:
     QString m_tmpDevicePath;
     QString m_tmpSpecificPath;
     QMap<QString, bool> m_bluetoothAdapters;
+    QTimer m_wirelessScanRetryTimer;
 
     void enableBluetooth(bool enable);
+    void scanRequestFailed(const QString &interface);
 
 };
 
